# -*- coding: utf-8 -*-
#######################################################################

###        ABACOM netPIO /incr3 board (Python 2 script)
###        
###      tested on Raspberry Pi Rev.2 (Linux)  ABACOM 11/2013 

###    other linux systems were not tested. Feedback wellcome.

#######################################################################

#######################################################################
### usage: netpio_incr3.py -a <host or IP> -p <UDP port>
#######################################################################

### example calls from linux command line:

###     python ./netpio_incr3.py --address 192.168.1.xxx --port 37157  
### or
###     python ./netpio_incr3.py -a myhostname.somewhere.com -p 37157 

### without below options only a read request is performed

### option [-c <CounterNo>] specifies a counter (1..6), ALL if not specified
### CounterNo 1..3 specifies    CNT 1..3
### CounterNo 4..6 specifies  Z-CNT 1..3

### option [-l <Value>] to load counter(s) with value
### option [-r] to reset counter(s)
### option [-d <DirByte>] to set port D direction, bit set = output
### option [-s <StatusByte>] to set port D status, bit set = (pulled) high
### option [-f <FreqByte>] adjust output freqency; -f 8  gives 1kHz

###################### required libraries  #######################

### These libs likely are already pre-installed on RaspPi...
import sys
import time
import getopt
import socket
import ctypes

# incr3 commands (Byte 1)
cmd_Read = 'R'
cmd_ResetCounter0 = 'A'
cmd_ResetCounter1 = 'B'
cmd_ResetCounter2 = 'C'
cmd_ResetZCounter0 = 'D'
cmd_ResetZCounter1 = 'E'
cmd_ResetZCounter2 = 'F'
cmd_LoadCounter0 = 'G'
cmd_LoadCounter1 = 'H'
cmd_LoadCounter2 = 'I'
cmd_LoadZCounter0 = 'J'
cmd_LoadZCounter1 = 'K'
cmd_LoadZCounter2 = 'L'
cmd_SetFreqOut = 'X'
cmd_SetPortDstatus = 'Y'
cmd_SetPortDdirection = 'Z'

# Wrap the (USB) request into UDP datagramm
# see netPIO datasheet http://www.abacom-online.de/div/FirmwareDocs/UK/ABACOM_netPIO.pdf
def wrapRequest(aRequest, responseLen, syncByte):
    # add leading "TXDATA" 
    aRequest.insert(0, "T")
    aRequest.insert(1, "X")
    aRequest.insert(2, "D")
    aRequest.insert(3, "A")
    aRequest.insert(4, "T")
    aRequest.insert(5, "A")
    # The USB request is this position
    # add length we exspect (or use syncbyte)
    aRequest.append(responseLen)
    aRequest.append(syncByte)

def CommandRequest(aHost, aPort, aCommand, aParam):
    request = bytearray() # incr3 request is 5 bytes long in any case
    # byte 1 - Command
    request.append(aCommand) 
    # byte 2..5 
    if aParam==None:
        for i in range(2,6):
            request.append(0) 
    else:
        request.append( (aParam & 0xFF)  )
        request.append( (aParam & (0xFF<<8) )>>8  )
        request.append( (aParam & (0xFF<<16) )>>16  )
        request.append( (aParam & (0xFF<<24) )>>24  )
    response=netpioincr3request(aHost, aPort, request)
    return response

# execute a netPIO /incr3 command via UDP
def netpioincr3request(aHost, aPort, request):

    # print the request bytes to debug
    # print "Request = ", str.join(" ", ("%02x" % i for i in request)) 

    response = bytearray()

    ### build the request...
    incr3ResponseLen = 21 # incr3 response is 21  bytes long
    UnusedSyncByte = 0 # we do not use a sync byte
    if len(request)<>5: # check the length, you in case...
        print "Bad request length, must be 5!"
        sys.exit;

    # Wrap the request into UDP datagramm 
    wrapRequest(request, incr3ResponseLen, UnusedSyncByte)

    ### ...send the request...
    # get a UDP socket
    UDP = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # (Internet, UDP)
    # and fire datagramm on UDP
    UDP.sendto(request, (aHost, aPort))

    ### ... and look for the response...
    UDP.settimeout(2.00) # float timeout value in seconds
    try:
        # no need to bind, we are bound by previous send 
        # sock.bind((UDP_IP, UDP_PORT))
        response, addr = UDP.recvfrom(incr3ResponseLen) # receive from UDP
    except socket.timeout:
        print "No response from ",aHost," (",aPort,")"
        sys.exit()
    return response

# printout the response  
def niceList(data):
    # data string to bytearray()
    response = bytearray(b"")
    for c in data: response.append(c)
     
    # print the response bytes to debug
    # print "Response = ", str.join(" ", ("%02x" % i for i in response)) 
 
    # response array is zero-based, but documentation is 1-based
    # so we match response[index] with documentation for better readability...
    response.insert(0,0x00) # we insert a dummy byte response[0] 
    # now we can decode easily...

    ##### Bytes 4...7: Counter1 
    CNT1 = (response[7] << 24) + (response[6] << 16) + (response[5] << 8) + response[4]
    ##### Bytes 8...11: Counter2 
    CNT2 = (response[11] << 24) + (response[10] << 16) + (response[9] << 8) + response[8]
    ##### Bytes 12...15: Counter3 
    CNT3 = (response[15] << 24) + (response[14] << 16) + (response[13] << 8) + response[12]
    ##### Bytes 16...17: Z-Counter1 
    ZCNT1 = (response[17] << 8) + response[16]
    ##### Bytes 18...19: Z-Counter2 
    ZCNT2 = (response[19] << 8) + response[18]
    ##### Bytes 20...21: Z-Counter3 
    ZCNT3 = (response[21] << 8) + response[20]
    print "unsigned counter values:"
    print " CNT1 =",str(CNT1).zfill(11),"  ZCNT1 =",str(ZCNT1).zfill(6) 
    print " CNT2 =",str(CNT2).zfill(11),"  ZCNT2 =",str(ZCNT2).zfill(6) 
    print " CNT3 =",str(CNT3).zfill(11),"  ZCNT3 =",str(ZCNT3).zfill(6) 

    print "signed counter values:"
    C1=ctypes.c_int32(CNT1).value
    C2=ctypes.c_int32(CNT2).value
    C3=ctypes.c_int32(CNT3).value
    Z1=ctypes.c_int16(ZCNT1).value
    Z2=ctypes.c_int16(ZCNT2).value
    Z3=ctypes.c_int16(ZCNT3).value
    print " CNT1 =",str(C1).zfill(11),"  ZCNT1 =",str(Z1).zfill(6) 
    print " CNT2 =",str(C2).zfill(11),"  ZCNT2 =",str(Z2).zfill(6) 
    print " CNT3 =",str(C3).zfill(11),"  ZCNT3 =",str(Z3).zfill(6) 

    # Byte 1,2,3: Digital ports 
    print "Digital port status 76543210:"
    print " Port B  =", "0x%02x" % response[1],' = ',bin(response[1])[2:].zfill(8)
    print " Port C  =", "0x%02x" % response[2],' = ',bin(response[2])[2:].zfill(8)
    print " Port D  =", "0x%02x" % response[3],' = ',bin(response[3])[2:].zfill(8)
 
### main method processes the command line ...
def main(argv):

    ### option [-c <CounterNo>] to specifiy a counter, all if not specified
    ### option [-l <Value>] to load counter(s) with value
    ### option [-r] to reset counter(s)
    ### option [-d <ByteValue>] to set port D direction
    ### option [-s <ByteValue>] to set port D status
    ### option [-f <ByteValue>] adjust output freqency

    # init the arg vars
    udpHostString= ''
    udpPortString = ''
    performSetRequest = False
    LoadValueString = ''
    StatDString = ''
    FreqString = ''
    DirDString = ''
    CounterString = ''
    ResetFlag=False

    ### Get string args from command line...
    try:
        argnameslong=["address=","port=","load=","counter=","dird=","statd=","freq="]
        opts, args = getopt.getopt(argv,"hra:p:l:c:d:s:f:",argnameslong)
    except getopt.GetoptError:
        print 'usage: netpio_incr3.py -a <host> -p <port>'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'usage: netpio_incr3.py -a <host> -p <port>'
            sys.exit()
        elif opt=='-r':
            ResetFlag=True
        elif opt in ("-a", "--address"):
            udpHostString = arg
        elif opt in ("-p", "--port"):
            udpPortString = arg
        elif opt in ("-l", "--load"):
            LoadValueString = arg
        elif opt in ("-c", "--counter"):
            CounterString = arg
        elif opt in ("-d", "--dird"):
            DirDString = arg
        elif opt in ("-s", "--statd"):
            StatDString = arg
        elif opt in ("-f", "--freq"):
            FreqString = arg
    ### We got the args now, so we can try to use it....

    ### Check UDP host from command line...
    if udpHostString=='': # no host specified in command line
        print 'No host specified!' 
        print 'usage: netpio_incr3.py -a <host> -p <port>'
        sys.exit()

    ### Now get UDP port from command line...
    if udpPortString=='': # no port specified in command line
        print 'No port specified, using default port 37155.' 
        udpPort = 37155
    else:
        try:
            udpPort = int(udpPortString)
        except ValueError:
            print 'Invalid port value!'
            sys.exit()
    if not (udpPort in range(1,65534)):
        print 'Port value out of range (must be 1..65534)'
        sys.exit()

    ### Now get LoadValue from command line...
    if LoadValueString=='': # not specified in command line
        LoadValue=0
    else:
        try:
            LoadValue = int(LoadValueString)
            if LoadValue==0:
               ResetFlag=True # as we do not load 0, we use reset instead
        except ValueError:
            LoadValue=0
            print 'Invalid load value!'
            sys.exit()

    ### Now get Counter from command line...
    if CounterString=='': # not specified in command line
        Counter=0
    else:
        try:
            Counter = int(CounterString)
        except ValueError:
            print 'Invalid counter value!'
            sys.exit()
        if not (Counter in range(1,7)):
            print 'Counter out of range (must be 1..6)'
            sys.exit()

    ### Hurray! We have host and port value.
    # see incr3 firmware docs for protokol details
    # http://www.abacom-online.de/div/FirmwareDocs/UK/Firmware_incr3.pdf

    ### Here we go...
    # call of device commands depending on specified args
    # the Port D functions...
    if FreqString<>'':  # -f Set output frequency @D6 ( = 9 kHz / (freq+1) )
        try:  
            Freq = int(FreqString)
            CommandRequest(udpHostString, udpPort, cmd_SetFreqOut, Freq)
            print "f out =", 9E3/(Freq+1),"Hz"
        except ValueError:
            print 'Invalid freq value!'
    if DirDString<>'': # - d Set Port D Direction (bit set = output)
        try:  
            DirD = int(DirDString)
            CommandRequest(udpHostString, udpPort, cmd_SetPortDdirection, DirD)
        except ValueError:
            print 'Invalid direction value!'
    if StatDString<>'': # - s Set Port D Direction (bit set = (pulled) high)
        try:  
            StatD = int(StatDString)
            CommandRequest(udpHostString, udpPort, cmd_SetPortDstatus, StatD)
        except ValueError:
            print 'Invalid status value!'
    # and counter(s) functions ...
    if Counter>0: # -c perform commands on specified counter 
        if ResetFlag: # -r Reset counter
           command=ord(cmd_ResetCounter0)+Counter-1 
           CommandRequest(udpHostString, udpPort, command, None)
        if LoadValue>0: # -l Load counter
           command=ord(cmd_LoadCounter0)+Counter-1 
           CommandRequest(udpHostString, udpPort, command, LoadValue)
    else: # no counter specified, perform commands on ALL counters
        for Counter in range(1,7):
            if ResetFlag: # -r Reset counters
                command=ord(cmd_ResetCounter0)+Counter-1 
                CommandRequest(udpHostString, udpPort, command, None)
            if LoadValue>0: # l Load counters
                command=ord(cmd_LoadCounter0)+Counter-1 
                CommandRequest(udpHostString, udpPort, command, LoadValue)

    ### Finally a read request and response printout 
    response=CommandRequest(udpHostString, udpPort, cmd_Read, None)
    niceList(response); # Printout the response

    ### finish!  
          
##############################################################
### call the main method with arguments from command line ...
if __name__ == "__main__":
   main(sys.argv[1:])
